#!/usr/bin/env node
'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _resolveFrom = require('resolve-from');

var _resolveFrom2 = _interopRequireDefault(_resolveFrom);

var _standalone = require('./standalone');

var _standalone2 = _interopRequireDefault(_standalone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cli = (0, _meow2.default)('\n    Usage\n        $ webfont [input] [options]\n\n    Input\n        Files(s) or glob(s).\n        If an input argument is wrapped in quotation marks, it will be passed to node-glob\n        for cross-platform glob support.\n\n    Options\n        --config                       Path to a specific configuration file (JSON, YAML, or CommonJS)\n                                       or the name of a module in `node_modules` that points to one.\n                                       If no `--config` argument is provided, webfont will search for\n                                       configuration  files in the following places, in this order:\n                                           - a `webfont` property in `package.json`\n                                           - a `.webfontrc` file (with or without filename extension:\n                                               `.json`, `.yaml`, and `.js` are available)\n                                           - a `webfont.config.js` file exporting a JS object\n                                       The search will begin in the working directory and move up the\n                                       directory tree until a configuration file is found.\n        -f, --font-name                The font family name you want, default: "webfont".\n        -h, --help                     Output usage information.\n        -v, --version                  Output the version number.\n        -r, --formats                  Only this formats generate.\n        -d, --dest                     Destination for generated fonts.\n        -t, --src-css-template         Path to custom template.\n        -c, --css-template-class-name  Class name in css template.\n        -p, --css-template-font-path   Font path in css template.\n        -n, --css-template-font-name   Font name in css template.\n        -s, --dest-css-template        Destination for generated css template.\n        --verbose                      Tell me everything!.\n\n    For "svgicons2svgfont":\n        --font-id                      The font id you want, default as "--font-name".\n        --style                        The font style you want.\n        --weight                       The font weight you want.\n        --fixed-width                  Creates a monospace font of the width of the largest input icon.\n        --center-horizontally          Calculate the bounds of a glyph and center it horizontally.\n        --normalize                    Normalize icons by scaling them to the height of the highest icon.\n        --height                       The outputted font height [MAX(icons.height)].\n        --round                        Setup the SVG path rounding [10e12].\n        --descent                      The font descent [0].\n        --ascent                       The font ascent [height - descent].\n        --start-unicode                The start unicode codepoint for unprefixed files [0xEA01].\n        --prepend-unicode              Prefix files with their automatically allocated unicode codepoint.\n        --metadata                     Content of the metadata tag.\n', {
    alias: {
        /* eslint-disable id-length */
        c: 'css-template-class-name',
        d: 'dest',
        f: 'font-name',
        h: 'help',
        n: 'css-template-font-name',
        p: 'css-template-font-path',
        r: 'formats',
        s: 'dest-css-template',
        t: 'src-css-template',
        v: 'version'
        /* eslint-enable id-length */
    },
    boolean: ['css', 'help', 'version', 'verbose', 'fixed-width', 'center-horizontally', 'normalize', 'prepend-unicode'],
    default: {
        config: false
    },
    string: ['font-name', 'dest', 'src-css-template', 'css-template-class-name', 'css-template-font-path', 'css-template-font-name', 'dest-css-template', 'font-id', 'style', 'weight', 'height', 'round', 'descent', 'ascent', 'start-unicode']
});

var optionsBase = {};

if (cli.flags.config) {
    // Should check these possibilities:
    //   a. name of a node_module
    //   b. absolute path
    //   c. relative path relative to `process.cwd()`.
    // If none of the above work, we'll try a relative path starting
    // in `process.cwd()`.
    optionsBase.configFile = (0, _resolveFrom2.default)(process.cwd(), cli.flags.config) || _path2.default.join(process.cwd(), cli.flags.config);
}

if (cli.flags.fontName) {
    optionsBase.fontName = cli.flags.fontName;
}

if (cli.flags.formats) {
    optionsBase.formats = cli.flags.formats;
}

if (cli.flags.dest) {
    optionsBase.dest = cli.flags.dest;
}

if (cli.flags.srcCssTemplate) {
    optionsBase.srcCssTemplate = cli.flags.srcCssTemplate;
}

if (cli.flags.cssTemplateClassName) {
    optionsBase.cssTemplateClassName = cli.flags.cssTemplateClassName;
}

if (cli.flags.cssTemplateFontPath) {
    optionsBase.cssTemplateFontPath = cli.flags.cssTemplateFontPath;
}

if (cli.flags.cssTemplateFontName) {
    optionsBase.cssTemplateFontName = cli.flags.cssTemplateFontName;
}

if (cli.flags.destCssTemplate) {
    optionsBase.destCssTemplate = cli.flags.destCssTemplate;
}

if (cli.flags.verbose) {
    optionsBase.verbose = cli.flags.verbose;
}

if (cli.flags.fontId) {
    optionsBase.fontId = cli.flags.fontId;
}

if (cli.flags.style) {
    optionsBase.style = cli.flags.style;
}

if (cli.flags.weight) {
    optionsBase.weight = cli.flags.weight;
}

if (cli.flags.fixedWidth) {
    optionsBase.fixedWidth = cli.flags.fixedWidth;
}

if (cli.flags.centerHorizontally) {
    optionsBase.centerHorizontally = cli.flags.centerHorizontally;
}

if (cli.flags.normalize) {
    optionsBase.normalize = cli.flags.normalize;
}

if (cli.flags.height) {
    optionsBase.height = cli.flags.height;
}

if (cli.flags.round) {
    optionsBase.round = cli.flags.round;
}

if (cli.flags.descent) {
    optionsBase.descent = cli.flags.descent;
}

if (cli.flags.ascent) {
    optionsBase.ascent = cli.flags.ascent;
}

if (cli.flags.startUnicode) {
    optionsBase.startUnicode = cli.flags.startUnicode;
}

if (cli.flags.prependUnicode) {
    optionsBase.prependUnicode = cli.flags.prependUnicode;
}

if (cli.flags.metadata) {
    optionsBase.metadata = cli.flags.metadata;
}

Promise.resolve().then(function () {
    return Object.assign({}, optionsBase, {
        files: cli.input
    });
}).then(function (options) {
    if (options.files.length === 0) {
        cli.showHelp();
    }

    if (!options.dest) {
        return Promise.reject(new Error('Require `--dest` options'));
    }

    if (options.srcCssTemplate && !options.destCssTemplate) {
        return Promise.reject(new Error('Require `--dest-css-template` options'));
    }

    if (options.destCssTemplate) {
        options.css = true;

        var extname = _path2.default.extname(options.destCssTemplate);

        options.cssFormat = extname ? extname.slice(1, extname.length) : 'css';
    }

    return (0, _standalone2.default)(options).then(function (result) {
        return Promise.resolve(result);
    }).then(function (result) {
        result.config = Object.assign({}, {
            dest: options.dest,
            destCssTemplate: options.destCssTemplate
        }, result.config);

        return Promise.resolve(result);
    });
}).then(function (result) {
    var fontName = result.config.fontName;

    return new Promise(function (resolve, reject) {
        (0, _mkdirp2.default)(_path2.default.resolve(result.config.dest), function (error) {
            if (error) {
                return reject(error);
            }

            return resolve();
        });
    }).then(function () {
        if (!result.css) {
            return Promise.resolve();
        }

        var cssTemplateDirectory = _path2.default.resolve(_path2.default.dirname(result.config.destCssTemplate));

        return new Promise(function (resolve, reject) {
            (0, _mkdirp2.default)(cssTemplateDirectory, function (error) {
                if (error) {
                    return reject(error);
                }

                return resolve();
            });
        });
    }).then(function () {
        return Promise.all(Object.keys(result).map(function (type) {
            if (type === 'config') {
                return Promise.resolve();
            }

            var content = result[type];
            var destFilename = null;

            if (type !== 'css') {
                destFilename = _path2.default.resolve(_path2.default.join(result.config.dest, fontName + '.' + type));
            } else {
                destFilename = _path2.default.resolve(result.config.destCssTemplate);
            }

            return new Promise(function (resolve, reject) {
                _fs2.default.writeFile(destFilename, content, function (error) {
                    if (error) {
                        return reject(new Error(error));
                    }

                    return resolve();
                });
            });
        }));
    }).then(function () {
        return Promise.resolve(result);
    });
}).catch(function (error) {
    console.log(error.stack); // eslint-disable-line no-console
    process.exit(error.code || 1); // eslint-disable-line no-process-exit
});