'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function () {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        files = _ref.files,
        configFile = _ref.configFile,
        _ref$fontName = _ref.fontName,
        fontName = _ref$fontName === undefined ? 'webfont' : _ref$fontName,
        _ref$formats = _ref.formats,
        formats = _ref$formats === undefined ? ['svg', 'ttf', 'eot', 'woff', 'woff2'] : _ref$formats,
        _ref$css = _ref.css,
        css = _ref$css === undefined ? false : _ref$css,
        _ref$cssFormat = _ref.cssFormat,
        cssFormat = _ref$cssFormat === undefined ? 'css' : _ref$cssFormat,
        _ref$cssTemplateClass = _ref.cssTemplateClassName,
        cssTemplateClassName = _ref$cssTemplateClass === undefined ? null : _ref$cssTemplateClass,
        _ref$cssTemplateFontP = _ref.cssTemplateFontPath,
        cssTemplateFontPath = _ref$cssTemplateFontP === undefined ? './' : _ref$cssTemplateFontP,
        _ref$cssTemplateFontN = _ref.cssTemplateFontName,
        cssTemplateFontName = _ref$cssTemplateFontN === undefined ? null : _ref$cssTemplateFontN,
        _ref$srcCssTemplate = _ref.srcCssTemplate,
        srcCssTemplate = _ref$srcCssTemplate === undefined ? null : _ref$srcCssTemplate,
        _ref$formatsOptions = _ref.formatsOptions,
        formatsOptions = _ref$formatsOptions === undefined ? {
        ttf: {
            copyright: null,
            ts: Math.round(Date.now() / 1000)
        }
    } : _ref$formatsOptions,
        _ref$metadataProvider = _ref.metadataProvider,
        metadataProvider = _ref$metadataProvider === undefined ? null : _ref$metadataProvider,
        _ref$fontId = _ref.fontId,
        fontId = _ref$fontId === undefined ? null : _ref$fontId,
        _ref$fontStyle = _ref.fontStyle,
        fontStyle = _ref$fontStyle === undefined ? '' : _ref$fontStyle,
        _ref$fontWeight = _ref.fontWeight,
        fontWeight = _ref$fontWeight === undefined ? '' : _ref$fontWeight,
        _ref$fixedWidth = _ref.fixedWidth,
        fixedWidth = _ref$fixedWidth === undefined ? false : _ref$fixedWidth,
        _ref$centerHorizontal = _ref.centerHorizontally,
        centerHorizontally = _ref$centerHorizontal === undefined ? false : _ref$centerHorizontal,
        _ref$normalize = _ref.normalize,
        normalize = _ref$normalize === undefined ? false : _ref$normalize,
        _ref$fontHeight = _ref.fontHeight,
        fontHeight = _ref$fontHeight === undefined ? null : _ref$fontHeight,
        _ref$round = _ref.round,
        round = _ref$round === undefined ? 10e12 : _ref$round,
        _ref$descent = _ref.descent,
        descent = _ref$descent === undefined ? 0 : _ref$descent,
        _ref$ascent = _ref.ascent,
        ascent = _ref$ascent === undefined ? undefined : _ref$ascent,
        _ref$startUnicode = _ref.startUnicode,
        startUnicode = _ref$startUnicode === undefined ? 0xEA01 : _ref$startUnicode,
        _ref$prependUnicode = _ref.prependUnicode,
        prependUnicode = _ref$prependUnicode === undefined ? false : _ref$prependUnicode,
        _ref$metadata = _ref.metadata,
        metadata = _ref$metadata === undefined ? null : _ref$metadata,
        _ref$verbose = _ref.verbose,
        verbose = _ref$verbose === undefined ? false : _ref$verbose;

    if (!files) {
        return Promise.reject(new Error('You must pass webfont a `files` glob'));
    }

    var glyphsData = null;

    return buildConfig({
        configFile: configFile
    }).then(function (buildedConfig) {
        var options = (0, _mergeDeep2.default)({}, {
            ascent: ascent,
            centerHorizontally: centerHorizontally,
            css: css,
            cssFormat: cssFormat,
            cssTemplateClassName: cssTemplateClassName,
            cssTemplateFontName: cssTemplateFontName,
            cssTemplateFontPath: cssTemplateFontPath,
            descent: descent,
            fixedWidth: fixedWidth,
            fontHeight: fontHeight,
            fontId: !fontId ? fontName : fontId,
            fontName: fontName,
            fontStyle: fontStyle,
            fontWeight: fontWeight,
            formats: formats,
            formatsOptions: formatsOptions,
            log: verbose ? console.log : function () {}, // eslint-disable-line no-console, no-empty-function
            metadata: metadata,
            metadataProvider: metadataProvider,
            normalize: normalize,
            prependUnicode: prependUnicode,
            round: round,
            srcCssTemplate: srcCssTemplate,
            startUnicode: startUnicode
        }, buildedConfig.config);

        return (0, _globby2.default)([].concat(files)).then(function (foundFiles) {
            var filteredFiles = foundFiles.filter(function (foundFile) {
                return _path2.default.extname(foundFile) === '.svg';
            });

            if (filteredFiles.length === 0) {
                return Promise.reject(new Error('Files glob patterns specified did not match any files'));
            }

            return Promise.resolve(filteredFiles);
        }).then(function (foundFiles) {
            glyphsData = getGlyphsData(foundFiles, options);

            if (!glyphsData) {
                throw new Error('Can\'t get glyphs data');
            }

            return glyphsData;
        }).then(function (returnedGlyphsData) {
            return svgIcons2svgFontFn(returnedGlyphsData, options);
        }).then(function (result) {
            return svg2ttfFn(result, options.formatsOptions.ttf);
        })
        // maybe add ttfautohint
        .then(function (result) {
            if (options.formats.indexOf('eot') === -1) {
                return Promise.resolve(result);
            }

            return ttf2eotFn(result);
        }).then(function (result) {
            if (options.formats.indexOf('woff') === -1) {
                return Promise.resolve(result);
            }

            return ttf2woffFn(result, {
                metadata: metadata
            });
        }).then(function (result) {
            if (options.formats.indexOf('woff2') === -1) {
                return Promise.resolve(result);
            }

            return ttf2woff2Fn(result);
        }).then(function (result) {
            if (!options.css) {
                return Promise.resolve(result);
            }

            if (!options.srcCssTemplate) {
                _nunjucks2.default.configure(_path2.default.join(__dirname, '../'));

                options.srcCssTemplate = _path2.default.join(__dirname, '../templates/template.' + options.cssFormat + '.njk');
            }

            var nunjucksOptions = (0, _mergeDeep2.default)({}, {
                glyphsData: glyphsData
            }, options, {
                className: options.cssTemplateClassName ? options.cssTemplateClassName : options.fontName,
                fontName: options.cssTemplateFontName ? options.cssTemplateFontName : options.fontName,
                fontPath: options.cssTemplateFontPath
            });

            result.css = _nunjucks2.default.render(_path2.default.resolve(options.srcCssTemplate), nunjucksOptions);

            return result;
        }).then(function (result) {
            if (options.formats.indexOf('svg') === -1) {
                delete result.svg;
            }

            if (options.formats.indexOf('ttf') === -1) {
                delete result.ttf;
            }

            result.config = options;

            return Promise.resolve(result);
        });
    }).catch(function (error) {
        throw error;
    });
};

var _stream = require('stream');

var _cosmiconfig = require('cosmiconfig');

var _cosmiconfig2 = _interopRequireDefault(_cosmiconfig);

var _metadata = require('svgicons2svgfont/src/metadata');

var _metadata2 = _interopRequireDefault(_metadata);

var _filesorter = require('svgicons2svgfont/src/filesorter');

var _filesorter2 = _interopRequireDefault(_filesorter);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _mergeDeep = require('merge-deep');

var _mergeDeep2 = _interopRequireDefault(_mergeDeep);

var _nunjucks = require('nunjucks');

var _nunjucks2 = _interopRequireDefault(_nunjucks);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _svg2ttf = require('svg2ttf');

var _svg2ttf2 = _interopRequireDefault(_svg2ttf);

var _svgicons2svgfont = require('svgicons2svgfont');

var _svgicons2svgfont2 = _interopRequireDefault(_svgicons2svgfont);

var _ttf2eot = require('ttf2eot');

var _ttf2eot2 = _interopRequireDefault(_ttf2eot);

var _ttf2woff = require('ttf2woff');

var _ttf2woff2 = _interopRequireDefault(_ttf2woff);

var _ttf2woff3 = require('ttf2woff2');

var _ttf2woff4 = _interopRequireDefault(_ttf2woff3);

var _xml2js = require('xml2js');

var _xml2js2 = _interopRequireDefault(_xml2js);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getGlyphsData(files, options) {
    var metadataProvider = options.metadataProvider || (0, _metadata2.default)({
        prependUnicode: options.prependUnicode,
        startUnicode: options.startUnicode
    });

    var sortedFiles = files.sort(function (fileA, fileB) {
        return (0, _filesorter2.default)(fileA, fileB);
    });
    var xmlParser = new _xml2js2.default.Parser();

    return Promise.all(sortedFiles.map(function (srcPath) {
        return new Promise(function (resolve, reject) {
            var glyph = _fs2.default.createReadStream(srcPath);
            var glyphContents = '';

            return glyph.on('error', function (glyphError) {
                return reject(glyphError);
            }).on('data', function (data) {
                glyphContents += data.toString();
            }).on('end', function () {
                xmlParser.parseString(glyphContents, function (error) {
                    if (error) {
                        return reject(error);
                    }

                    var glyphData = {
                        contents: glyphContents,
                        srcPath: srcPath
                    };

                    return resolve(glyphData);
                });
            });
        }).then(function (glyphData) {
            return new Promise(function (resolve, reject) {
                metadataProvider(glyphData.srcPath, function (error, metadata) {
                    if (error) {
                        return reject(error);
                    }

                    glyphData.metadata = metadata;

                    return resolve(glyphData);
                });
            });
        });
    }));
}

function svgIcons2svgFontFn(glyphsData, options) {
    var result = {
        svg: ''
    };

    return new Promise(function (resolve, reject) {
        var fontStream = (0, _svgicons2svgfont2.default)({
            ascent: options.ascent,
            centerHorizontally: options.centerHorizontally,
            descent: options.descent,
            fixedWidth: options.fixedWidth,
            fontHeight: options.fontHeight,
            fontId: options.fontId,
            fontName: options.fontName,
            fontStyle: options.fontStyle,
            fontWeight: options.fontWeight,
            log: options.log,
            metadata: options.metadata,
            normalize: options.normalize,
            round: options.round
        }).on('finish', function () {
            return resolve(result);
        }).on('data', function (data) {
            result.svg += data;
        }).on('error', function (error) {
            return reject(error);
        });

        glyphsData.forEach(function (glyphData) {
            var glyphStream = new _stream.Readable();

            glyphStream.push(glyphData.contents);
            glyphStream.push(null);

            glyphStream.metadata = glyphData.metadata;

            fontStream.write(glyphStream);
        });

        fontStream.end();
    });
}

function svg2ttfFn(result, options) {
    return new Promise(function (resolve) {
        // eslint-disable-next-line node/no-deprecated-api
        result.ttf = new Buffer((0, _svg2ttf2.default)(result.svg.toString(), {
            copyright: options.copyright,
            ts: options.ts,
            version: options.version
        }).buffer);

        return resolve(result);
    });
}

function ttf2eotFn(result) {
    return new Promise(function (resolve) {
        // eslint-disable-next-line node/no-deprecated-api
        result.eot = new Buffer((0, _ttf2eot2.default)(new Uint8Array(result.ttf)).buffer);

        return resolve(result);
    });
}

function ttf2woffFn(result, options) {
    return new Promise(function (resolve) {
        // eslint-disable-next-line node/no-deprecated-api
        result.woff = new Buffer((0, _ttf2woff2.default)(new Uint8Array(result.ttf), options).buffer);

        return resolve(result);
    });
}

function ttf2woff2Fn(result) {
    return new Promise(function (resolve) {
        result.woff2 = (0, _ttf2woff4.default)(result.ttf);

        return resolve(result);
    });
}

function buildConfig(options) {
    if (!options.configFile) {
        return Promise.resolve({});
    }

    var cosmiconfigOptions = {
        // Allow extensions on rc filenames
        rcExtensions: true
    };

    var configPath = process.cwd();

    if (options.configFile) {
        configPath = _path2.default.resolve(process.cwd(), options.configFile);
    }

    return (0, _cosmiconfig2.default)('webfont', cosmiconfigOptions).load(null, configPath).then(function (result) {
        if (!result) {
            return Promise.reject(new Error('No configuration found'));
        }

        return Promise.resolve({
            config: result.config
        });
    });
}